# -*- coding: utf-8 -*-

"""Top-level package for CM4AI Generate Hierarchy."""

__author__ = 'Clara Hu'
__email__ = 'mhu@ucsd.edu'
__version__ = '0.1.0a2'
__repo_url__ = 'https://github.com/idekerlab/cellmaps_generate_hierarchy'
__description__ = 'A tool to generate hierarchies from protein to protein interaction networks'



#! /usr/bin/env python

import os
import logging
import time
import json
from datetime import date
from tqdm import tqdm
from cellmaps_utils import constants
from cellmaps_utils import logutils
from cellmaps_utils.provenance import ProvenanceUtil
import cellmaps_generate_hierarchy
from cellmaps_generate_hierarchy.exceptions import CellmapsGenerateHierarchyError


logger = logging.getLogger(__name__)


class CellmapsGenerateHierarchy(object):
    """
    Runs steps necessary to create PPI from embedding and to
    generate a hierarchy
    """
    def __init__(self, outdir=None,
                 inputdir=None,
                 ppigen=None,
                 hiergen=None,
                 name=None,
                 organization_name=None,
                 project_name=None,
                 provenance_utils=ProvenanceUtil(),
                 input_data_dict=None):
        """
        Constructor

        :param outdir: Directory to create and put results in
        :type outdir: str
        :param ppigen: PPI Network Generator object, should be a subclass
        :type ppigen: :py:class:`~cellmaps_generate_hierarchy.ppi.PPINetworkGenerator`
        :param hiergen: Hierarchy Generator object, should be a subclass
        :type hiergen: :py:class:`~cellmaps_generate_hierarchy.CXHierarchyGenerator`
        :param name:
        :param organization_name:
        :param project_name:
        :param provenance_utils:
        """
        logger.debug('In constructor')
        if outdir is None:
            raise CellmapsGenerateHierarchyError('outdir is None')
        self._outdir = os.path.abspath(outdir)
        self._inputdir = inputdir
        self._start_time = int(time.time())
        self._ppigen = ppigen
        self._hiergen = hiergen
        self._name = name
        self._project_name = project_name
        self._organization_name = organization_name
        self._input_data_dict = input_data_dict
        self._provenance_utils = provenance_utils

    def _create_rocrate(self):
        """
        Creates rocrate for output directory

        :raises CellMapsProvenanceError: If there is an error
        """
        logger.debug('Registering rocrate with FAIRSCAPE')
        name, proj_name, org_name = self._provenance_utils.get_name_project_org_of_rocrate(self._inputdir)

        if self._name is not None:
            name = self._name

        if self._organization_name is not None:
            org_name = self._organization_name

        if self._project_name is not None:
            proj_name = self._project_name
        try:
            self._provenance_utils.register_rocrate(self._outdir,
                                                    name=name,
                                                    organization_name=org_name,
                                                    project_name=proj_name)
        except TypeError as te:
            raise CellmapsGenerateHierarchyError('Invalid provenance: ' + str(te))
        except KeyError as ke:
            raise CellmapsGenerateHierarchyError('Key missing in provenance: ' + str(ke))

    def _register_software(self):
        """
        Registers this tool

        :raises CellMapsImageEmbeddingError: If fairscape call fails
        """
        self._softwareid = self._provenance_utils.register_software(self._outdir,
                                                                    name=cellmaps_generate_hierarchy.__name__,
                                                                    description=cellmaps_generate_hierarchy.__description__,
                                                                    author=cellmaps_generate_hierarchy.__author__,
                                                                    version=cellmaps_generate_hierarchy.__version__,
                                                                    file_format='.py',
                                                                    url=cellmaps_generate_hierarchy.__repo_url__)

    def _register_computation(self, generated_dataset_ids=[]):
        """
        # Todo: added inused dataset, software and what is being generated
        :return:
        """
        logger.debug('Getting id of input rocrate')
        input_dataset_id = self._provenance_utils.get_id_of_rocrate(self._inputdir)
        self._provenance_utils.register_computation(self._outdir,
                                                    name=cellmaps_generate_hierarchy.__name__ + ' computation',
                                                    run_by=str(self._provenance_utils.get_login()),
                                                    command=str(self._input_data_dict),
                                                    description='run of ' + cellmaps_generate_hierarchy.__name__,
                                                    used_software=[self._softwareid],
                                                    used_dataset=[input_dataset_id],
                                                    generated=generated_dataset_ids)

    def get_ppi_network_dest_file(self, ppi_network):
        """
        Gets the path where the PPI network should be written to

        :param ppi_network: PPI Network
        :type ppi_network: :py:class:`ndex2.nice_cx_network.NiceCXNetwork`
        :return: Path on filesystem to write the PPI network
        :rtype: str
        """
        cutoff = ppi_network.get_network_attribute('cutoff')['v']
        return os.path.join(self._outdir, constants.PPI_NETWORK_PREFIX +
                            '_cutoff_' + str(cutoff) + constants.CX_SUFFIX)

    def get_hierarchy_dest_file(self, hierarchy):
        """
        Gets the path where the hierarchy should be written to.
        Current implementation appends `_#cutoff` to filename
        where `#` is the cutoff value used in making the PPI network

        :param hierarchy: Hierarchy Network
        :type hierarchy: :py:class:`ndex2.nice_cx_network.NiceCXNetwork`
        :return: Path on filesystem to write Hierarchy Network
        :rtype: str
        """
        cutoff = 'unknown'
        try:
            cutoff = hierarchy.get_network_attribute('cutoff')['v']
        except Exception as e:
            logger.error('Unable to get cutoff from hierarchy network ' + str(e))

        return os.path.join(self._outdir, constants.HIERARCHY_NETWORK_PREFIX +
                            '_cutoff_' + str(cutoff) + constants.CX_SUFFIX)

    def run(self):
        """
        Runs CM4AI Generate Hierarchy


        :return:
        """
        exitcode = 99
        try:
            logger.debug('In run method')

            if os.path.isdir(self._outdir):
                raise CellmapsGenerateHierarchyError(self._outdir + ' already exists')

            if not os.path.isdir(self._outdir):
                os.makedirs(self._outdir, mode=0o755)

            logutils.setup_filelogger(outdir=self._outdir,
                                      handlerprefix='cellmaps_image_embedding')
            logutils.write_task_start_json(outdir=self._outdir,
                                           start_time=self._start_time,
                                           data={'commandlineargs': self._input_data_dict},
                                           version=cellmaps_generate_hierarchy.__version__)

            self._create_rocrate()

            self._register_software()
            generated_dataset_ids = []
            for ppi_network in tqdm(self._ppigen.get_next_network(), desc='Generating hierarchy'):

                logger.debug('Writing PPI network ' + str(ppi_network.get_name()))
                # write PPI to filesystem
                ppi_out_file = self.get_ppi_network_dest_file(ppi_network)
                with open(ppi_out_file, 'w') as f:
                    json.dump(ppi_network.to_cx(), f)

                # register ppi network file with fairscape
                data_dict = {'name': os.path.basename(ppi_out_file) + ' PPI network file',
                             'description': 'PPI Network file',
                             'data-format': 'CX',
                             'author': cellmaps_generate_hierarchy.__name__,
                             'version': cellmaps_generate_hierarchy.__version__,
                             'date-published': date.today().strftime('%m-%d-%Y')}
                generated_dataset_ids.append(self._provenance_utils.register_dataset(self._outdir,
                                                                                     source_file=ppi_out_file,
                                                                                     data_dict=data_dict))

                # generate hierarchy
                logger.info('Creating hierarchy')
                hierarchy = self._hiergen.get_hierarchy(ppi_network)
                hierarchy_out_file = self.get_hierarchy_dest_file(hierarchy)
                with open(hierarchy_out_file, 'w') as f:
                    json.dump(hierarchy.to_cx(), f)
                    # register ppi network file with fairscape
                    data_dict = {'name': os.path.basename(hierarchy_out_file) + ' Hierarchy network file',
                                 'description': 'Hierarchy network file',
                                 'data-format': 'CX',
                                 'author': cellmaps_generate_hierarchy.__name__,
                                 'version': cellmaps_generate_hierarchy.__version__,
                                 'date-published': date.today().strftime('%m-%d-%Y')}
                    generated_dataset_ids.append(self._provenance_utils.register_dataset(self._outdir,
                                                                                         source_file=hierarchy_out_file,
                                                                                         data_dict=data_dict))

            self._register_computation(generated_dataset_ids=generated_dataset_ids)
            exitcode = 0
        finally:
            logutils.write_task_finish_json(outdir=self._outdir,
                                            start_time=self._start_time,
                                            status=exitcode)

        return exitcode



#! /usr/bin/env python

import argparse
import sys
import logging
import logging.config

from cellmaps_utils import logutils
from cellmaps_utils import constants
import cellmaps_generate_hierarchy
from cellmaps_generate_hierarchy.ppi import CosineSimilarityPPIGenerator
from cellmaps_generate_hierarchy.hierarchy import CDAPSHierarchyGenerator
from cellmaps_generate_hierarchy.runner import CellmapsGenerateHierarchy

logger = logging.getLogger(__name__)


CO_EMBEDDINGDIR='--coembedding_dir'


def _parse_arguments(desc, args):
    """
    Parses command line arguments

    :param desc: description to display on command line
    :type desc: str
    :param args: command line arguments usually :py:func:`sys.argv[1:]`
    :type args: list
    :return: arguments parsed by :py:mod:`argparse`
    :rtype: :py:class:`argparse.Namespace`
    """
    parser = argparse.ArgumentParser(description=desc,
                                     formatter_class=constants.ArgParseFormatter)
    parser.add_argument('outdir', help='Output directory')
    parser.add_argument(CO_EMBEDDINGDIR, required=True,
                        help='Directory where coembedding was run')
    parser.add_argument('--name',
                        help='Name of this run, needed for FAIRSCAPE. If '
                             'unset, name value from specified '
                             'by --coembedding_dir directory will be used')
    parser.add_argument('--organization_name',
                        help='Name of organization running this tool, needed '
                             'for FAIRSCAPE. If unset, organization name specified '
                             'in --coembedding_dir directory will be used')
    parser.add_argument('--project_name',
                        help='Name of project running this tool, needed for '
                             'FAIRSCAPE. If unset, project name specified '
                             'in --coembedding_dir directory will be used')
    parser.add_argument('--logconf', default=None,
                        help='Path to python logging configuration file in '
                             'this format: https://docs.python.org/3/library/'
                             'logging.config.html#logging-config-fileformat '
                             'Setting this overrides -v parameter which uses '
                             ' default logger. (default None)')
    parser.add_argument('--verbose', '-v', action='count', default=0,
                        help='Increases verbosity of logger to standard '
                             'error for log messages in this module. Messages are '
                             'output at these python logging levels '
                             '-v = ERROR, -vv = WARNING, -vvv = INFO, '
                             '-vvvv = DEBUG, -vvvvv = NOTSET (default no '
                             'logging)')
    parser.add_argument('--version', action='version',
                        version=('%(prog)s ' +
                                 cellmaps_generate_hierarchy.__version__))

    return parser.parse_args(args)


def main(args):
    """
    Main entry point for program

    :param args: arguments passed to command line usually :py:func:`sys.argv[1:]`
    :type args: list

    :return: return value of :py:meth:`cellmaps_generate_hierarchy.runner.CellmapsGenerateHierarchy.run`
             or ``2`` if an exception is raised
    :rtype: int
    """
    desc = """
    Version {version}

    Takes a coembedding file {coembedding_file} file from {coembedding_dir} directory that
    is in TSV format and generates an interaction network
    from which a hierarchy is derived. 
    
    Format of {coembedding_file} where 1st line is header:
    
    ''\t1\t2\t3\t4\t5...1024
    GENESYMBOL\tEMBEDDING1\tEMBEDDING2...
    
    Example:
    
            1       2       3       4       5
    AAAS    -0.35026753     -0.1307554      -0.046265163    0.3758623       0.22126552

    """.format(version=cellmaps_generate_hierarchy.__version__,
               coembedding_file=constants.CO_EMBEDDING_FILE,
               coembedding_dir=CO_EMBEDDINGDIR)
    theargs = _parse_arguments(desc, args[1:])
    theargs.program = args[0]
    theargs.version = cellmaps_generate_hierarchy.__version__

    try:
        logutils.setup_cmd_logging(theargs)

        ppigen = CosineSimilarityPPIGenerator(embeddingdir=theargs.coembedding_dir)

        hiergen = CDAPSHierarchyGenerator()
        return CellmapsGenerateHierarchy(outdir=theargs.outdir,
                                         inputdir=theargs.coembedding_dir,
                                         ppigen=ppigen,
                                         hiergen=hiergen,
                                         input_data_dict=theargs.__dict__).run()
    except Exception as e:
        logger.exception('Caught exception: ' + str(e))
        return 2
    finally:
        logging.shutdown()


if __name__ == '__main__':  # pragma: no cover
    sys.exit(main(sys.argv))




import os
import math
import pandas as pd
import numpy as np
import ndex2
from cellmaps_utils import music_utils
from cellmaps_utils import constants
from cellmaps_generate_hierarchy.exceptions import CellmapsGenerateHierarchyError


class PPINetworkGenerator(object):
    """
    Base class for objects that generate
    Protein to Protein interaction networks
    """
    def __init__(self):
        """
        Constructor
        """
        pass

    def get_next_network(self):

        """
        Gets next protein to protein interaction network

        :return: Network
        :rtype: :py:class:`ndex2.nice_cx_network.NiceCXNetwork`
        """
        raise NotImplementedError('subclasses need to implement')


class CosineSimilarityPPIGenerator(PPINetworkGenerator):
    """
    Takes Embedding file of format:

    .. code-block::

        ID # # # #

    Where ID is gene and #'s is embedding vector
    """
    def __init__(self, embeddingdir=None,
                 cutoffs=[0.01]):
        """
        Constructor
        """
        super().__init__()
        if embeddingdir is None:
            raise CellmapsGenerateHierarchyError('embeddingdir is None')

        self._embeddingfile = os.path.join(embeddingdir,
                                           constants.CO_EMBEDDING_FILE)
        self._cutoffs = cutoffs

    def _get_ppi_dataframe(self):
        """

        :return:
        """
        z = pd.read_table(self._embeddingfile, sep='\t', index_col=0)
        sim_mat = music_utils.cosine_similarity_scaled(z)
        keep = np.triu(np.ones(sim_mat.shape)).astype(bool)
        sim_mat = sim_mat.where(keep)

        pairs = sim_mat.stack().reset_index().rename(columns={'level_0': constants.PPI_EDGELIST_GENEA_COL,
                                                              'level_1': constants.PPI_EDGELIST_GENEB_COL,
                                                              0: constants.WEIGHTED_PPI_EDGELIST_WEIGHT_COL})

        pairs = pairs[pairs[constants.PPI_EDGELIST_GENEA_COL] != pairs[constants.PPI_EDGELIST_GENEB_COL]]
        return pairs.sort_values(constants.WEIGHTED_PPI_EDGELIST_WEIGHT_COL, ascending=False)

    def get_next_network(self):
        """
        Gets all the edges

        :param cutoff: Fraction of top edges to keep
                       0.01 means 1% 0.5 means 50%
        :type cutoff: float
        :return: Network
        :rtype: :py:class:`ndex2.nice_cx_network.NiceCXNetwork`
        """
        df = self._get_ppi_dataframe()
        for cutoff in self._cutoffs:
            df_cutoff = df.iloc[0:math.ceil(cutoff*len(df))]
            net = ndex2.create_nice_cx_from_pandas(df_cutoff,
                                                   source_field=constants.PPI_EDGELIST_GENEA_COL,
                                                   target_field=constants.PPI_EDGELIST_GENEB_COL,
                                                   edge_attr=[constants.WEIGHTED_PPI_EDGELIST_WEIGHT_COL])
            net.set_name('cellmaps_generate_hierarchy PPI ' + str(cutoff) + ' cutoff')
            net.set_network_attribute(name='description',
                                      values='Protein to Protein Interaction\n'
                                             'network generated by cellmaps_generate_hierarchy\n'
                                             'tool from embedding XXX where top ' +
                                             str(round(cutoff*100.0)) + '% of interactions sorted by\n')
            net.set_network_attribute(name='cutoff', values=str(cutoff))
            #             # Todo add generated by
            #  author and other information
            yield net





import logging
import cdapsutil


logger = logging.getLogger(__name__)

class CXHierarchyGenerator(object):
    """
    Base class for generating hierarchy
    that is output in CX format following
    CDAPS style
    """
    def __init__(self):
        """
        Constructor
        """
        pass

    def get_hierarchy(self):
        """
        Gets hierarchy

        # Todo: flip return object to HCX object

        :return:
        :rtype: :py:class:`ndex2.nice_cx_network.NiceCXNetwork`
        """
        raise NotImplementedError('Subclasses need to implement')


class CDAPSHierarchyGenerator(CXHierarchyGenerator):
    """
    Generates hierarchy using CDAPS
    """
    def __init__(self):
        """
        Constructor
        """
        super().__init__()

    def get_hierarchy(self, network):
        """

        :return:
        """
        cd = cdapsutil.CommunityDetection(runner=cdapsutil.ServiceRunner())

        hierarchy = cd.run_community_detection(network,
                                               algorithm='hidef')
        cutoff = 'unknown'
        try:
            cutoff = network.get_network_attribute('cutoff')['v']
        except Exception as e:
            logger.error(str(e))
        hierarchy.set_network_attribute(name='cutoff', values=str(cutoff))
        return hierarchy









#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `cellmaps_generate_hierarchy` package."""

import os
import shutil
import tempfile
import unittest
from cellmaps_generate_hierarchy.runner import CellmapsGenerateHierarchy


class TestCellmapsgeneratehierarchyrunner(unittest.TestCase):
    """Tests for `cellmaps_generate_hierarchy` package."""

    def setUp(self):
        """Set up test fixtures, if any."""

    def tearDown(self):
        """Tear down test fixtures, if any."""

    def test_constructor(self):
        """Tests constructor"""
        temp_dir = tempfile.mkdtemp()
        try:
            myobj = CellmapsGenerateHierarchy(outdir=os.path.join(temp_dir, 'out'))
            self.assertIsNotNone(myobj)
        finally:
            shutil.rmtree(temp_dir)




#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `CDAPSHierarchyGenerator`."""

import os

import shutil
import tempfile
import unittest
import ndex2
from cellmaps_utils import constants
from cellmaps_generate_hierarchy.hierarchy import CDAPSHierarchyGenerator


class TestCDAPSHierarchyGenerator(unittest.TestCase):
    """Tests for `CDAPSHierarchyGenerator`."""

    def setUp(self):
        """Set up test fixtures, if any."""

    def tearDown(self):
        """Tear down test fixtures, if any."""

    def test_get_hierarchy(self):
        gen = CDAPSHierarchyGenerator()

        net = ndex2.nice_cx_network.NiceCXNetwork()
        net.set_name('Test')
        node_a = net.create_node('A')
        node_b = net.create_node('B')
        node_c = net.create_node('C')
        node_d = net.create_node('D')

        net.create_edge(edge_source=node_a, edge_target=node_b)
        net.create_edge(edge_source=node_a, edge_target=node_c)
        net.create_edge(edge_source=node_b, edge_target=node_c)
        net.create_edge(edge_source=node_b, edge_target=node_d)

        hierarchy = gen.get_hierarchy(net)

        self.assertEqual(1, len(hierarchy.get_nodes()))


